<?php

/**
 * Plugin Name: BitKassa
 * Plugin URI: https://www.bitkassa.nl/en/webshop-plugin
 * Description: BitKassa payments for WooCommerce
 * Version: 0.4.2
 * Author: Martijn Buurman / Roy van Zanten / Djuri Baars
 * Author URI: https://www.bitkassa.nl
 * License: GPL2 http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

/**
 * Check if WooCommerce is active
 **/
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

    function declareWooBitKassa()
    {
        if (!class_exists('WC_Payment_Gateways')) {
            return;
        }

        require 'bitkassa-v1api-wrapper.php';

        class WC_BitKassa extends WC_Payment_Gateway
        {

            protected $bitkassa;

            public function __construct()
            {
                $this->id = 'bitkassa';
                $this->has_fields = false;

                $this->init_form_fields();

                $this->init_settings();

                $this->title = $this->settings['title'];
                $this->description = $this->settings['description'];
                $this->icon = trailingslashit( WP_PLUGIN_URL ) . plugin_basename( dirname( __FILE__ ) ) . '/assets/bitcoin.svg';


                $this->bitkassa = new BitKassaV1ApiWrapper($this->get_option('merchant_id'), $this->get_option('secret_api_key'));

                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
                add_action('woocommerce_thankyou', array($this, 'thankyou_page'), 10, 1);               
                add_action('woocommerce_api_bitkassacb', array($this, 'webhook'));
            }

            public function init_form_fields()
            {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __('Enable/Disable', 'woothemes'),
                        'type' => 'checkbox',
                        'label' => __('Enable BitKassa Payment', 'woothemes'),
                        'default' =>  'yes'
                    ),
                    'title' => array(
                        'title' => __('Title', 'woothemes'),
                        'type' => 'text',
                        'description' => __('This controls the title which the user sees during checkout.', 'woothemes'),
                        'default' => __('Bitcoin', 'woothemes')
                    ),
                    'description' => array(
                        'title' => __('Customer Message', 'woothemes'),
                        'type' => 'textarea',
                        'description' => __('Message to explain how the customer will be paying for the purchase.', 'woothemes'),
                        'default' => 'You will be redirected to BitKassa to complete your purchase.'
                    ),
                    'instructions'       => array(
                        'title'       => __('Instructions', 'woocommerce'),
                        'type'        => 'textarea',
                        'description' => __('Instructions that will be added to the thank you page.', 'woocommerce'),
                        'desc_tip'    => true,
                    ),
                    'icon_css'       => array(
                        'title'       => __('Icon CSS', 'woocommerce'),
                        'type'        => 'textarea',
                        'description' => __('Inline CSS styles for gateway icon.', $this->id),
                        'desc_tip'    => true,
                        'default' => 'max-height: 24px; float: right;'

                    ),
                    'merchant_id' => array(
                        'title' => __('Merchant ID', 'wc_bitkassa'),
                        'type' => 'text',
                        'description' => __('Enter the Merchant ID you created in the BitKassa platform'),
                    ),
                    'secret_api_key' => array(
                        'title' => __('Secret API Key', 'wc_bitkassa'),
                        'type' => 'text',
                        'description' => __('Enter the Secret API Key you created in the BitKassa platform'),
                    ),
                    'pending_payment_status' => array(
                        'title' => __('Pending Payment Status', 'wc_bitkassa'),
                        'type'        => 'select',
                        'class'       => 'wc-enhanced-select',
                        'description' => __('The payment status while not yet completed.', 'wc_bitkassa'),
                        'default'     => 'wc-pending',
                        'desc_tip'    => true,
                        'options'     => wc_get_order_statuses()
                    )
                );
            }

            public function admin_options()
            {
                echo '<h3>' . _e('Bitcoin Payment', 'woothemes') . '</h3>';
                echo '<p>' . _e('Allows bitcoin payments via BitKassa', 'woothemes') . '</p>';
                echo '<table class="form-table">';
                $this->generate_settings_html();
                echo '</table>';
            }

            function payment_fields()
            {
                if ($this->description) echo wpautop(wptexturize($this->description));
            }

            function thankyou_page()
            {
                try{
                    $params = $this->bitkassa->getParams();

                    if ($params['payment_status'] == BitKassaV1ApiWrapper::STATUS_CANCELLED) {
                        echo wpautop(wptexturize("Payment cancelled"));

                        wc_add_notice("Payment cancelled", 'error');
                        wp_redirect(wc_get_checkout_url(), '301');

                        return array(
                            'result' => 'failure',
                            'redirect'    => wc_get_checkout_url()
                        );
                    }  
                        
                } catch (Exception $e) {
                   wp_add_notice("Error parsing BitKassa respone");
                
                    return array(
                        'result' => 'failure',
                    );
                }
            }

            public function get_icon() {
                $icon_url  = $this->icon;
                $icon_html = sprintf('<img src="%s" alt="%s" style="%s"/>', $icon_url, $this->method_title, $this->get_option('icon_css') );
            
                return apply_filters( 'woocommerce_gateway_icon', $icon_html, $this->id );
            }

            function process_payment($order_id)
            {

                global $woocommerce;

                $order = wc_get_order($order_id);
                // Mark as on-hold (we're awaiting the coins)
                $order->update_status($this->get_option('pending_payment_status'), __('Awaiting payment notification from BitKassa', 'woothemes'));

                // redirect to the order confirmation page after leaving www.bitkassa.nl
                $redirect = $this->get_return_url($order);
                // bitkassa should report transaction updates to this url
                $report_url = get_option('siteurl') . "/wc-api/bitkassacb/?order=" . $order_id;

                // options for the API call
                $options = array(
                    'description' => get_bloginfo('name') . ' order #' . $order->get_id(),
                    'return_url' => $redirect,
                    'update_url' => $report_url,
                    'currency' => $order->get_currency(),
                    'amount' => $order->get_total() * 100
                );

                // call the bitkassa API and start the payment
                $bitkassaOrder = $this->bitkassa->startPayment($options);

                if ($bitkassaOrder['success'] && $bitkassaOrder['payment_url']) {

                    // save the bitkassa transaction id with this order
                    update_post_meta(
                        $order->get_id(),
                        'bitkassa_payment_id',
                        $bitkassaOrder['payment_id']
                    );

                    return array(
                        'result' => 'success',
                        'redirect' => $bitkassaOrder['payment_url']
                    );
                } else {
                    wc_add_notice(__('Error creating BitKassa invoice. Please try again or try another payment method.'), 'error');
                    return array(
                        'result' => 'failure'
                    );
                }
            }

            function webhook()
            {
                // search for the order_id with the corresponding bitkassa transaction id
                $order = wc_get_order($_GET["order"]);

                $payment_id = $order->get_meta('bitkassa_payment_id');

                $bitkassaTransactionStatus = $this->bitkassa->getPaymentStatus($payment_id);

                if ($bitkassaTransactionStatus['success']) {
                    switch ($bitkassaTransactionStatus['payment_status']) {
                        case BitKassaV1ApiWrapper::STATUS_OPEN:
                            break;
                        case BitKassaV1ApiWrapper::STATUS_PAID:
                            WC()->cart->empty_cart();
                            $order->payment_complete();
                            break;
                        case BitKassaV1ApiWrapper::STATUS_EXPIRED:
                        case BitKassaV1ApiWrapper::STATUS_CANCELLED:
                            $order->update_status('cancelled');
                            break;
                    }
                }
            }

            function callback($order_id)
            {
                // search for the order_id with the corresponding bitkassa transaction id
                $order = wc_get_order($order_id);

                $payment_id = $order->get_meta('bitkassa_payment_id');

                $bitkassaTransactionStatus = $this->bitkassa->getPaymentStatus($payment_id);

                if ($bitkassaTransactionStatus['success']) {
                    switch ($bitkassaTransactionStatus['payment_status']) {
                        case BitKassaV1ApiWrapper::STATUS_OPEN:
                            break;
                        case BitKassaV1ApiWrapper::STATUS_PAID:
                            WC()->cart->empty_cart();
                            $order->payment_complete();
                            break;
                        case BitKassaV1ApiWrapper::STATUS_EXPIRED:
                        case BitKassaV1ApiWrapper::STATUS_CANCELLED:
                            $order->update_status('cancelled');
                            break;
                    }
                }
            }

            function is_available()
            {
                $supported_currencies = array("eur", "btc");
                if (!in_array(strtolower(get_woocommerce_currency()), $supported_currencies))
                    return false;
                return true;
            }
        }
    }

    function add_bitkassa_gateway($methods)
    {
        $methods[] = 'WC_BitKassa';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'add_bitkassa_gateway');

    add_action('plugins_loaded', 'declareWooBitKassa', 0);
}
